<?php

namespace Fazed\NyaaRssParser\Factories;

use Fazed\NyaaRssParser\Contracts\HttpClient\ResponseContract;
use Fazed\NyaaRssParser\Contracts\Factories\ResponseFactoryContract;
use Fazed\NyaaRssParser\Contracts\Factories\EntryModelFactoryContract;

class ResponseFactory implements ResponseFactoryContract
{
    /**
     * @var EntryModelFactoryContract
     */
    private $entryModelFactory;

    /**
     * ResponseFactory constructor.
     *
     * @param EntryModelFactoryContract $entryModelFactory
     */
    public function __construct(EntryModelFactoryContract $entryModelFactory)
    {
        $this->entryModelFactory = $entryModelFactory;
    }

    /**
     * {@inheritdoc}
     */
    public function make($responseData)
    {
        // Prepare a fresh response instance.
        $response = app(ResponseContract::class);

        // Return an empty response when no data was provided.
        if (null === $responseData) {
            return $response;
        }

        // Fetch the Nyaa-namespace from the document, without
        // it we can't interpret all data for the entries.
        $namespace = $responseData->getDocNamespaces()['nyaa'];

        // Convert the items to Entry instances.
        $entries = array_map(function (\SimpleXMLElement $entry) use ($namespace) {
            return $this->entryModelFactory->make(
                array_map(function ($value) {
                    return (string) $value;
                }, array_merge((array) $entry->children(), (array) $entry->children($namespace)))
            );
        }, $responseData->xpath('channel/item'));

        // Set the entries on the response instance.
        return $response->setEntries($entries);
    }
}
