<?php

namespace Fazed\NyaaRssParser\HttpClient;

use Httpful\Mime;
use Httpful\Request;
use Fazed\NyaaRssParser\Contracts\HttpClient\ClientContract;
use Illuminate\Contracts\Config\Repository as ConfigRepository;
use Fazed\NyaaRssParser\Contracts\Factories\ResponseFactoryContract;

class Client implements ClientContract
{
    /**
     * @var ConfigRepository
     */
    private $configRepository;

    /**
     * @var ResponseFactoryContract
     */
    private $responseFactory;

    /**
     * Client constructor.
     *
     * @param ConfigRepository $configRepository
     * @param ResponseFactoryContract $responseFactory
     */
    public function __construct(
        ConfigRepository $configRepository,
        ResponseFactoryContract $responseFactory
    )
    {
        $this->configRepository = $configRepository;
        $this->responseFactory = $responseFactory;
    }

    /**
     * {@inheritdoc}
     */
    public function dispatchRequest(array $queries = [])
    {
        try {
            return $this->responseFactory->make(
                Request::getQuick(
                    $this->prepareDispatchUrl($queries),
                    Mime::XML
                )->body
            );
        } catch (\Exception $e) {
            return $this->responseFactory->make(null);
        }
    }

    /**
     * Prepare a dispatchable URL, combining data
     * from the configuration and given queries.
     *
     * @param  array $queries
     * @return string
     */
    protected function prepareDispatchUrl(array $queries)
    {
        // Merge the given queries with the additional
        // queries provided in the configuration file.
        $queries = array_merge(
            $this->configRepository->get('nyaa-rss-parser.base_queries', []),
            $queries
        );

        // Filter any empty values from the queries.
        $queries = array_filter($queries, function ($queryValue) {
            return ! empty($queryValue);
        });

        // Recursively encode the query values just
        // in case to avoid incorrect parsing.
        $queries = array_map(function ($value) {
            return rawurlencode($value);
        }, $queries);

        // Create the query string from the query
        // array we've build in the step above.
        $queries = http_build_query($queries, null, '&', PHP_QUERY_RFC3986);

        // Assemble the URL using the base URL from
        // the configuration and prepared queries.
        return $this->configRepository->get('nyaa-rss-parser.base_url', '')
            . ( ! empty($queries) ? '?' . $queries : '');
    }
}
