<?php

namespace Fazed\NyaaRssParser\Models;

use ArrayAccess;
use JsonSerializable;
use Illuminate\Contracts\Support\Jsonable;
use Illuminate\Contracts\Support\Arrayable;
use Fazed\NyaaRssParser\Contracts\Models\DataModelContract;

abstract class DataModel implements DataModelContract, Arrayable, ArrayAccess, JsonSerializable, Jsonable
{
    /**
     * DataModel constructor.
     *
     * @param array $properties
     */
    protected function __construct(array $properties = [])
    {
        $this->fill($properties);
    }

    /**
     * {@inheritdoc}
     */
    public function fill(array $properties)
    {
        foreach ($properties as $property=>$value) {
            $this->offsetSet($property, $value);
        }

        return $this;
    }

    /**
     * {@inheritdoc}
     */
    public function offsetExists($offset)
    {
        return property_exists($this, $offset);
    }

    /**
     * {@inheritdoc}
     */
    public function offsetGet($offset)
    {
        if ($this->offsetExists($offset)) {
            return $this->$offset;
        }

        return null;
    }

    /**
     * {@inheritdoc}
     */
    public function offsetSet($offset, $value)
    {
        if ($this->offsetExists($offset)) {
            $this->$offset = $value;
        }
    }

    /**
     * {@inheritdoc}
     */
    public function offsetUnset($offset)
    {
        if ($this->offsetExists($offset)) {
            unset($this->$offset);
        }
    }

    /**
     * {@inheritdoc}
     */
    public function toArray()
    {
        return get_object_vars($this);
    }

    /**
     * {@inheritdoc}
     */
    public function jsonSerialize()
    {
        return $this->toArray();
    }

    /**
     * {@inheritdoc}
     */
    public function toJson($options = 0)
    {
        return json_encode($this->jsonSerialize(), $options);
    }
}
